import csv
from synch2jira.issue import Issue

class IssueCSV():
     
    @staticmethod
    def generate_csv_data(issue_list, file_path):
            fieldnames = Issue.__annotations__.keys()
            print(fieldnames)

            with open(file_path, 'w', newline='') as csvfile:
                writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
                writer.writeheader()
                for issue in issue_list:
                    issue_dict = issue.__dict__
                    writer.writerow(issue_dict)


import csv
from dataclasses import dataclass
from datetime import datetime, timezone


import config
from synch2jira.issue_S2 import IssueS2




@dataclass
class IssueWokflowJSON():
   
    def __init__(self, issueKey, status, updated, from_time, to_time):
        self.from_time = from_time
        self.issueKey = issueKey
        self.updated = updated
        self.status = status
        self.to_time = to_time

   
    @staticmethod
    def get_issue_from_json(issue_key, json_data):
        for issue in json_data : 
            if issue.get('key') == issue_key:
                return issue
        return 
    
    @staticmethod
    def get_workflow(json_data,issue_key,status):
        issue_data = IssueWokflowJSON.get_issue_from_json(issue_key,json_data)
        changelog = issue_data.get("changelog", [])
        histories = changelog.get("histories", [])
        from_time = []
        to_time = []
        for history in histories:
            created = history.get("created", [])
            print(created)
            items = history.get('items', [])
            for item in items:
                field = item['field']
                if field == "status":
                    #if item['fromString'] == status:
                        #from_time.append(created)
                    if item['toString'] == status:
                        to_time.append(created)
                min_from_time = None if from_time == [] else min(from_time)
        max_to_time = None if to_time == [] else max(to_time)
        print(to_time,max_to_time)
        return max_to_time
    
    @staticmethod
    def get_lead_time(json_data,issue_key, first_status, last_status):
        workflow1 = IssueWokflowJSON.get_workflow(json_data, issue_key,first_status)
        workflow2 = IssueWokflowJSON.get_workflow(json_data, issue_key,last_status)
        print(workflow1,workflow2)
        return workflow1,workflow2


    @staticmethod
    def save_result_in_csv(result, entete):
        filename = config.csv_file
        with open(filename, mode="w", newline="") as file:
            writer = csv.writer(file)
            writer.writerow(entete)
            for row in result:
                writer.writerow(row)
        print(f"Les données ont été écrites dans {filename}")

   


from __future__ import annotations
from dataclasses import dataclass, field
from datetime import date, datetime, timedelta
from typing import List, Optional
import config
import csv


@dataclass
class Issue:
    summary: str
    description: str
    updated: date
    status: str
    status_updated: date
    miror: Optional[Issue] = None

    issue_key :str = ""                 
    issue_id : str = ""
    statuscategorychangedate : str = ""
    issuetypeid : str = ""
    issuetypedescription : str = ""
    issuetypename : str  = ""
    issuetypesubtask : bool = None
    issuetypehierarchyLevel : str  = ""
    timespend : str    = ""
    projectid : str  = ""
    projectkey : str  = ""
    projectname : str = ""
    projectTypeKey : str  = ""
    aggregatetimespent : str  = ""
    resolution : str  = ""
    resolutiondate : str  = ""
    workratio : str  = ""
    watchCount : str  = ""
    isWatching : bool = None
    lastViewed : str = ""
    created : str = ""
    priorityname : str  = ""
    priorityid : str  = ""
    labelsnumber : str  = ""
    assignee : str  = ""
    #updated : str = ""
    statusname : str  = ""
    statuscategoryname : str  = ""
    timeoriginalestimate : str = ""
    #description : str  = ""
    security : str  = ""
    aggregatetimeestimate : str = ""
    #summary : str  = ""
    creatoremailAddress : str  = ""
    creatorname : str = ""
    subtasksnumber : str  = ""
    reportername : str  = ""
    reporteremail : str  = ""
    duedate : str = ""
    votes : str  = ""
    workflow_start_time : str = ""
    workflow_end_time : str = ""


    def json_to_issue(json_data):
        fields = json_data.get("fields", {})
        return Issue(
    issue_key=json_data.get("key"),
    issue_id=json_data.get("id"),
    statuscategorychangedate=fields.get("statuscategorychangedate"),
    issuetypeid=fields["issuetype"].get("id"),
    issuetypedescription=fields["issuetype"].get("description"),
    issuetypename=fields["issuetype"].get("name"),
    issuetypesubtask=fields["issuetype"].get("subtask"),
    issuetypehierarchyLevel=fields["issuetype"].get("hierarchyLevel"),
    timespend=fields.get("timespent"),
    projectid=fields["project"].get("id"),
    projectkey=fields["project"].get("key"),
    projectname=fields["project"].get("name"),
    projectTypeKey=fields["project"].get("projectTypeKey"),
    aggregatetimespent=fields.get("aggregatetimespent"),
    resolution=fields.get("resolution"),
    resolutiondate=fields.get("resolutiondate"),
    workratio=fields.get("workratio"),
    watchCount=fields["watches"].get("watchCount"),
    isWatching=fields["watches"].get("isWatching"),
    lastViewed=fields.get("lastViewed"),
    created=fields["created"].replace(" ", ""),
    priorityname=fields["priority"].get("name"),
    priorityid=fields["priority"].get("id"),
    labelsnumber=len(fields.get("labels", [])),
    assignee=fields.get("assignee"),
    updated=fields["updated"].replace(" ", ""),
    statusname=fields["status"].get("name"),
    statuscategoryname=fields["status"]["statusCategory"].get("name"),
    timeoriginalestimate=fields.get("timeoriginalestimate"),
    description="",
    #description=fields["description"]["content"][0].get("text"),
    security=fields.get("security"),
    aggregatetimeestimate=fields.get("aggregatetimeestimate"),
    summary=fields.get("summary"),
    creatoremailAddress=fields["creator"].get("emailAddress"),
    creatorname=fields["creator"].get("displayName"),
    subtasksnumber=len(fields.get("subtasks", [])),
    reportername=fields["reporter"].get("displayName"),
    reporteremail=fields["reporter"].get("emailAddress"),
    duedate=fields.get("duedate", ""),
    votes=fields["votes"].get("votes"),
)

    def generate_csv_data(issue_list, file_path):
        fieldnames = Issue.__annotations__.keys()

        with open(file_path, 'w', newline='') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()
            for issue in issue_list:
                issue_dict = issue.__dict__
                writer.writerow(issue_dict)

    def delete(self):
        pass

    def get(self):
        pass

    def all(self):
        pass

    def update(self):
        pass

    def synchroniser_status_miroir_avec_status(self):
        if self.miror:
            print(self.miror.status)
            self.miror.status = config.status_dict_S2_to_S1[self.status]
            self.miror.update()
            return True
        return False

    @classmethod
    def convertirS_enS_(cls, issue):
        return cls(summary=issue.summary, description=issue.description, status=issue.status, updated=datetime.now())

    @staticmethod
    def convertir_en_format_date(date_to_convert):
        if isinstance(date_to_convert, datetime):
            date_to_convert = date_to_convert.strftime('%Y-%m-%d %H:%M:%S.%f')
        date = datetime.strptime(date_to_convert, '%Y-%m-%d %H:%M:%S.%f')
        offset = date.utcoffset()
        date_formattee = date + (offset if offset else timedelta(hours=0))
        return date_formattee

    @staticmethod
    def convertir_str_date_en_datetime(date_to_convert):
        date_without_timezone = date_to_convert[:-5]
        date_without_microseconds = datetime.strptime(date_without_timezone, '%Y-%m-%dT%H:%M:%S.%f')
        microseconds = int(date_to_convert[-5:])
        return date_without_microseconds.replace(microsecond=microseconds)

    @classmethod
    def all_attribut_id_et_updated(cls):
        issues = cls.all_filtre_id_et_updated()
        issues_list = []
        for issue in issues:
            issues_list.append(cls(summary="", description="", updated=issue.updated, status=""))
        return issues_list

    @staticmethod
    def date_S1_entre_date_derniere_synchro_et_date_S2(date_S1, date_S2):
        last_synchronized_date = Issue.convertir_str_date_en_datetime(config.last_synchronized_data_in_S2)
        date_S1_format_datetime = Issue.convertir_en_format_date(date_S1)
        date_S2_format_datetime = Issue.convertir_str_date_en_datetime(date_S2)

        return last_synchronized_date <= date_S1_format_datetime < date_S2_format_datetime

    def synch_summary(self):
        if self.miror:
            print(self.miror.summary)
            self.summary = self.miror.summary
            self.update()
            return True
        return False

    def synch_description(self):
        if self.miror:
            self.description = self.miror.description
            self.update()
            return True
        return False

    def synch_status(self):
        if self.miror:
            try:
                self.change_issue_status(config.status_dict_S1_to_S2[self.miror.status])
                return True
            except Exception as e:
                try:
                    self.change_issue_status(config.status_dict_S2_to_S1[self.miror.status])
                    return True
                except Exception as e:
                    return False

    def synchroniser_status_avec_status_miroir(self):
        if self.miror:
            print(self.miror)
            self.change_issue_status(config.status_dict_S1_to_S2[self.miror.status])
            return True
        return False

    def synchroniser_status_miroir_status(self):
        if self.miror:
            print(self.miror.status)
            self.miror.status = config.status_dict_S2_to_S1[self.status]
            self.miror.update()
            return True
        return False

    def synch_summary_from_miror(self):
        self.summary = self.miror.summary
        # Il faut juste qu'il update summary et non pas description aussi et tout !
        self.update()

    def synch_summary_to_miror(self):
        self.miror.summary = self.summary
        self.update()

    def synch_description_from_miror(self):
        self.description = self.miror.description
        self.update()

    def synch_description_to_miror(self):
        self.miror.description = self.description
        self.update()

    @staticmethod
    def issue1_factory(summary, description, updated, status):
        module_name = config.module_to_use
        class_name = config.class_to_use
        module = __import__(module_name, fromlist=[class_name])
        cls = getattr(module, class_name)
        return cls(summary, description, updated, status)

    def get_workflow_duration_between_to_status(self, status1, status2):
        for history in self.workflow_history:
            if history["to_state"] == status1:
                begin = history["status_change_time"]
            if history["to_state"] == status2:
                end = history["status_change_time"]
        return begin, end



import pandas as pd
import matplotlib
matplotlib.use('Agg')

import matplotlib.pyplot as plt


def csv_to_dataframe(file_path):
    df = pd.read_csv(file_path)
    return df

def genarate_creation_resolution_figure(dataframe):
    dataframe['created'] = pd.to_datetime(dataframe['created'])
    dataframe['resolutiondate'] = pd.to_datetime(dataframe['resolutiondate'])
    # Trier les données par date de création
    dataframe.sort_values(by='created', inplace=True)
    df_filtered = dataframe.dropna(subset=['resolutiondate'])
    print(df_filtered)
    plt.figure(figsize=(12, 6))
    plt.plot(df_filtered.index, df_filtered['created'], label='Date de création', color='blue')
    plt.plot(df_filtered.index, df_filtered['resolutiondate'], label='Date de résolution', color='green')
    plt.xlabel('ticket')
    plt.ylabel('dates creation_resolution')
    plt.title('Tendances creation_resolution des tickets ')
    plt.xticks(rotation=45)  
    plt.legend()
    plt.tight_layout()  
    plt.savefig('creation_resolution.png')
    plt.show()

def str_df_to_dt_df(dataframe):
    dataframe['created'] = pd.to_datetime(dataframe['created'],utc=True)
    dataframe['resolutiondate'] = pd.to_datetime(dataframe['resolutiondate'],utc=True)
    dataframe = dataframe.dropna(subset=['resolutiondate','created'])
    return dataframe



def get_creation_resolution_time_statistics(dataframe):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['creation_resolution_time'] = dataframe.apply(lambda row: row['resolutiondate'] - row['created'], axis=1)
    average_resolution_time = dataframe['creation_resolution_time'].mean()
    min_resolition_time = dataframe['creation_resolution_time'].min()
    max_resolution_time = dataframe['creation_resolution_time'].max()
    print("Différence de temps moyenne entre création et résolution : ", average_resolution_time)
    print("difference max de temps de resolution",dataframe['creation_resolution_time'].max(),dataframe['creation_resolution_time'].min())
    print(dataframe['creation_resolution_time'].describe())
    return dataframe['creation_resolution_time'].describe()
 
def get_month_statistics(dataframe ):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['created_month'] = dataframe['created'].dt.month
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created'] #
    dataframe = dataframe[['created_month','resolution_time']]
    # statistiques groupées par mois
    statistics_by_month = dataframe.groupby('created_month')['resolution_time'].describe()
    print(statistics_by_month)


    #statistics_by_resolution_month = dataframe.groupby('created_month')['resolution_time'].agg(['mean', 'median', 'std', 'min', 'max'])
    #print(statistics_by_resolution_month)
    return statistics_by_month

def get_period_statistics(dataframe, period):
    dataframe = str_df_to_dt_df(dataframe)
    
    if period == 'week':
        dataframe['period'] = dataframe['created'].dt.isocalendar().week
    elif period == 'month':
        dataframe['period'] = dataframe['created'].dt.month
    elif period == 'year':
        dataframe['period'] = dataframe['created'].dt.year
    else:
        raise ValueError("Période non valide. Veuillez spécifier 'day', 'month' ou 'year'.")
    
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
    
    
    dataframe = dataframe[['period', 'resolution_time']]
    
    statistics_by_period = dataframe.groupby('period')['resolution_time'].describe()
    
    return statistics_by_period


def get_statistics_by_issue_type(dataframe):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
    statistics_by_issue_type = dataframe.groupby('issuetypename')['resolution_time'].describe()

    return statistics_by_issue_type

def get_statistics_by_field(dataframe,field):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
    statistics_by_issue_field = dataframe.groupby(field)['resolution_time'].describe()

    return statistics_by_issue_field

def get_double_group_by_statistics(dataframe,period):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
    if period == 'week':
        dataframe['period'] = dataframe['created'].dt.isocalendar().week
    elif period == 'month':
        dataframe['period'] = dataframe['created'].dt.month
    elif period == 'year':
        dataframe['period'] = dataframe['created'].dt.year
    else:
        raise ValueError("Période non valide. Veuillez spécifier 'day', 'month' ou 'year'.")
    
    statistics = dataframe.groupby(['period',"issuetypename"])['resolution_time'].describe()
    for line in statistics :
        print(line)
        print(statistics[line])
    return statistics


file_path = 'issue.csv'  
dataframe = csv_to_dataframe(file_path)
# df_created_solved = dataframe[['issue_key','created','resolutiondate']]
# #print(df_created_solved)
# #print(type(get_creation_resolution_time_statistics(dataframe)))
# print(get_month_statistics(dataframe))

#print(get_month_statistics(dataframe))
#genarate_creation_resolution_figure(dataframe)
#print(get_statistics_by_field(dataframe,'creatorname'))
#print(get_statistics_by_field(dataframe,'issuetypename'))
#print(get_statistics_by_field(dataframe,'projectname'))

#print(get_statistics_by_field(dataframe,'assignee'))
result = get_double_group_by_statistics(dataframe,'week')
#,issuetypeid,,issuetypename,issuetypesubtask,issuetypehierarchyLevel,timespend,projectid,,projectname,projectTypeKey,aggregatetimespent,resolutiondate,workratio,watchCount,isWatching,lastViewed,created,priorityname,priorityid,labelsnumber,assignee,statusname,statuscategoryname,,,aggregatetimeestimate,creatoremailAddress,creatorname,subtasksnumber,reportername,reporteremail,duedate,votes,workflow_start_time,workflow_end_time


